/*
   DocMatch.java -- a document fond in the word database

   $Id: DocMatch.java,v 1.1 1999/05/24 19:41:57 eg Exp $

   Copyright (c) 1995-1999 The ht://Dig Group
   Copyright (C) 1999, Edwin Glaser

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

package htsearch;

import java.util.Vector;
import java.util.Date;
import java.text.DateFormat;

/**
 * DocMatch -- a document fond in the word database
 *
 *
 */

public class DocMatch 
{
  static int excerpt_length = 300;
  static String start_ellipses = "<b><tt> ...</tt></b>";
  static String end_ellipses = "<b><tt> ...</tt></b>";

  static DateFormat dateformat = DateFormat.getDateInstance();

  int score;
  public int Percent;
  int id;
  int anchor;
  byte[] key;
  Vector searchWords;
  
  byte[] urlKey;
  String URL;
  DocumentRef ref;
  
  /**
   * Get the value of urlKey.
   * @return Value of urlKey.
   */
  public byte[] getUrlKey() {return urlKey;}
  public String getURL() {return URL;}
  
  /**
   * Set the value of urlKey.
   * @param v  Value to assign to urlKey.
   */
  public void setUrlKey(byte[]  v) 
  {
	urlKey = v;
	URL = HtURLCodec.instance().decode(new String(v));
  }
  
  /**
   * Get the value of ref.
   * @return Value of ref.
   */
  public DocumentRef getRef() {return ref;}

  public String getAnchor()
  {
	String  a = null;
	if ( anchor > 0 && ref.docAnchors != null && ref.docAnchors.length >= anchor )
	{
		a = "#" + ref.docAnchors[anchor-1];
	}
	return a;
  }

  public String getTime()
  {
	return dateformat.format(new Date(((long)ref.docTime) * 1000));
  }

  public String getHead()
  {
	return ref.docHead;
  }

  public String getMetaDsc()
  {
	return ref.docMetaDsc;
  }

  public String getTitle()
  {
	if ( ref.docTitle == null )
	{
		String ret = URL;
		int pos = ret.lastIndexOf("/");
		if ( pos > 0 ) ret = ret.substring(pos + 1);
		return "[" + ret + "]";
	}
	return ref.docTitle;
  }

  public String  getSize()
  {
	return String.valueOf(ref.docSize);
  }

  public String getURLAnchor()
  {
	String  a = URL;
	if ( anchor > 0 && ref.docAnchors != null && ref.docAnchors.length >= anchor )
	{
		a = a + "#" + ref.docAnchors[anchor-1];
	}
	return a;
  }

  public String getExcerpt()
  {
	return getExcerpt(false);
  }

  public String getExcerpt(boolean useMeta)
  {
	String head = null;
	String lower = null;
	String text = "";

	if ( useMeta )
	{
		head = ref.docMetaDsc;
	}
	if ( head == null )
	{
		head = ref.docHead;
	}
	lower = head.toLowerCase();
	
	match m = findFirstWord(lower, 0);
	int first = (m == null) ? 0 : m.pos;

	int headPos = head.length() - 1;
	int start = first - excerpt_length / 2;
	if ( start <= 0 )
	{
		start = 0;
	}
	else
	{
		text = text + start_ellipses;
		while ( start < headPos && Character.isLetter(head.charAt(start)) )
		{
			start++;
		}
	}
	int end = start + excerpt_length;
	while ( end < headPos && Character.isLetter(head.charAt(end)) )
	{
		end++;
	}
	if ( end > headPos )
	{
		text = text + hilight(head.substring(start));
	}
	else
	{
		text = text + hilight(head.substring(start, end));
		text = text + end_ellipses;
	}
	return text;
  }

  String hilight(String str)
  {
	String lower = str.toLowerCase();
	String result = "";
	String urlanchor = getAnchor();
	String prefix;
	String postfix;
	if ( urlanchor != null)
	{
		prefix = "<a href=\"" + URL + urlanchor + "\"><strong>";
		postfix = "</strong></a>";
	}
	else
	{
		prefix = "<strong>";
		postfix = "</strong>";
	}

	int start = 0;
	match m;
	while ( (m = findFirstWord(lower, start)) != null )
	{
		result = result + str.substring(start, m.pos)
		  + prefix + str.substring(m.pos, m.pos + m.length) + postfix;
		start = m.pos + m.length;
	}
	result = result + str.substring(start);
	return result;
  }

  class match
  {
	int pos;
	int witch;
	int length;
  }
  
  match findFirstWord(String head, int start)
  {
	int first = -1;
	int witch = -1;
	int length = -1;
	for ( int i = searchWords.size() - 1; i >= 0; i-- )
	{
		String test = (String)searchWords.elementAt(i);
		int pos = head.indexOf(test, start);
		if ( pos >= 0 && (first == -1 || pos < first) )
		{
			first = pos;
			witch = i;
			length = test.length(); 
		}
	}
	if ( first >= 0 )
	{
		match m = new match();
		m.pos = first;
		m.witch = witch;
		m.length = length;
		return m;
	}
	return null;
  }

  /**
   * Set the value of ref.
   * @param v  Value to assign to ref.
   */
  public void setRef(DocumentRef  v) {this.ref = v;}

  static int htoi(byte[] entry, int pos)
  {
	return ((((int) entry[pos + 3]) & 0xff) << 28)
	  + ((((int) entry[pos + 2]) & 0xff) << 16)
	  + ((((int) entry[pos + 1]) & 0xff) << 8)
	  + (((int) entry[pos]) & 0xff);
  }

  public static void addDbEntry(byte[] entry, Vector res, Vector searchWords)
  {
	int items = entry.length / 20;
	int pos = 0;
	for ( int i = 0; i < items; i++ )
	{
		int count = htoi(entry, pos);
		pos += 4;
		int id = htoi(entry, pos);
		pos += 4;
		int weight = htoi(entry, pos);
		pos += 4;
		int anchor = htoi(entry, pos);
		pos += 4;
		int location = htoi(entry, pos);
		pos += 4;
		res.addElement(new DocMatch(id, weight, anchor, searchWords));
	}
  }


  public DocMatch(int id, int score, int anchor, Vector searchWords)
  {
	this.id = id;
	this.key = String.valueOf(id).getBytes();
	this.score = score;
	this.anchor = anchor;
	this.searchWords = searchWords;
  }

  public String toString()
  {
	return "DocMatch(" + id + ", " + anchor + ", " + Percent + "%, " + ref + ")";
  }
} // DocMatch
