/*
   HtSearch.java -- the search engine

   $Id: HtSearch.java,v 1.1 1999/05/24 19:41:57 eg Exp $

   Copyright (c) 1995-1999 The ht://Dig Group
   Copyright (C) 1999, Edwin Glaser

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

package htsearch;

import java.util.Vector;
import java.util.StringTokenizer;
import java.text.Collator;
/**
 * HtSearch -- the search engine
 *
 *
 */

public class HtSearch 
{
  static String WordsName = ".words.db";
  static String IndexName = ".docs.index";
  static String DocName = ".docdb";

  static int  minimum_word_length = 3;

  static final int SortByScore = 0;
  static final int SortByScoreRev = 1;
  static final int SortByTime = 2;
  static final int SortByTimeRev = 3;
  static final int SortByTitle = 4;
  static final int SortByTitleRev = 5;

  Vector tokens;
  Vector words;

  Parser parser;

  Vector result;
  boolean hasRef;
  int maxScore;

  public HtSearch(String dbdir)
    throws Exception
  {
	parser = new Parser (dbdir + WordsName, dbdir + IndexName, dbdir + DocName);
  }

  public boolean hadError()
  {
	if ( parser == null ) return true;
	return parser.hadError();
  }

  public String getErrorMessage()
  {
	if ( parser == null ) return "not initialized";
	return parser.getErrorMessage();
  }

  public int search (int sortTyp)
  {
	result = null;
	hasRef = false;
	maxScore = 1;

	result = parser.parse(tokens, words);
	if ( parser.hadError() )
	{
		return -1;
	}

	if ( result.size() == 0 )
	{
		return 0;
	}

	for ( int i = result.size() - 1; i >= 0; i-- )
	{
		DocMatch m = (DocMatch) result.elementAt(i);
		maxScore = Math.max(maxScore, m.score);
		if ( sortTyp > 1 )
		{
			parser.fetchDocumentRef(m);
			if ( parser.hadError() )
			{
				return -1;
			}
			hasRef = true;
		}
	}

	sort(sortTyp);
	return result.size();
  }

  public int getSize()
  {
	return result == null ? 0 : result.size();
  }

  public DocMatch get(int pos, boolean fetchUrl, boolean fetchRef)
  {
	if ( result == null || result.size() <= pos ) return null;
	DocMatch m = (DocMatch) result.elementAt(pos);

	if ( fetchUrl && !hasRef && m.getURL() == null )
	{
		parser.fetchURL(m);
		if ( parser.hadError() )
		{
			return null;
		}
	}
	if ( fetchRef && !hasRef && m.getRef() == null )
	{
		parser.fetchDocumentRef(m);
		if ( parser.hadError() )
		{
			return null;
		}
	}
	m.Percent = Math.max(m.score * 100 / maxScore, 1);
	return m;
  }

  static Collator collate = Collator.getInstance();

  boolean bevore(int sortType, DocMatch n, DocMatch o)
  {
	if ( sortType == SortByScore )
	{
		return n.score < o.score;
	}
	else if ( sortType == SortByScoreRev )
	{
		return n.score > o.score;
	}
	else if ( sortType == SortByTime )
	{
		return n.ref.docTime < o.ref.docTime;
	}
	else if ( sortType == SortByTimeRev )
	{
		return n.ref.docTime > o.ref.docTime;
	}
	else if ( sortType == SortByTitle )
	{
		return collate.compare(n.getTitle(), o.getTitle()) > 0;
	}
	else if ( sortType == SortByTitleRev )
	{
		return collate.compare(n.getTitle(), o.getTitle()) < 0;
	}
	
	return false;
  }

  void sort(int sortTyp) // FIXME: needs a real sorting algorithm
  {
	Vector tmp = result;
	result = new Vector();

	int last = tmp.size();
	for ( int i = 0; i < last; i++ )
	{
		DocMatch m = (DocMatch) tmp.elementAt(i);
		int j = 0;
		while ( j < i && bevore(sortTyp, m, (DocMatch) result.elementAt(j)) )
		{
			j++;
		}
		result.insertElementAt(m, j);
	}
  }

  public boolean setupWords(String allWords, boolean useBool, boolean useAnd)
  {
	Vector originalPattern = new Vector();
	Vector tempWords = new Vector();
	StringTokenizer st = new StringTokenizer(allWords, useBool ? " \t()" : " \t", true);
	while ( st.hasMoreTokens() )
	{
		String word = st.nextToken();
		if ( word.equals(" ") || word.equals("\t"))
		{
			continue;
		}
		word = word.toLowerCase();
		if ( useBool && word.equals("and") )
		{
			tempWords.addElement(new WeightWord("&", -1.0));
		}
		else if ( useBool && word.equals("or") )
		{
			tempWords.addElement(new WeightWord("|", -1.0));
		}
		else if ( useBool && word.equals("not") )
		{
			tempWords.addElement(new WeightWord("!", -1.0));
		}
        else
		{
			originalPattern.addElement(word);
			// HtStripPunctuation(word);
			WeightWord	ww = new WeightWord(word, 1.0);
			if (// !badWords.IsValid(word) ||
				word.length() < minimum_word_length )
			{
				ww.isIgnore = true;
			}
			tempWords.addElement(ww);
		}
	}

	if (useBool)
	{
		if (!parser.checkSyntax(tempWords))
		{
			return false;
		}
	}
	else
	{
		tempWords = convertToBoolean(tempWords, useAnd);
	}

	//
	// We need to assign weights to the words according to the search_algorithm
	// configuration attribute.
	// For algorithms other than exact, we need to also do word lookups.
	//

	tokens = tempWords;
	words = originalPattern;
	return  true;
  }

  Vector convertToBoolean(Vector words, boolean do_and)
  {
    if (words.size() == 0)
	  return words;

    Vector list = new Vector();
    list.addElement(words.elementAt(0));
    for (int i = 1; i < words.size(); i++)
    {
		if (do_and)
		  list.addElement(new WeightWord("&", -1.0));
		else
		  list.addElement(new WeightWord("|", -1.0));
		list.addElement(words.elementAt(i));
    }
	return list;
  }

  public static void main (String[] Argv)
  {
	String Dir = "/usr/local/htdig/db/db";

	try
	{
		HtSearch ht = new HtSearch (Dir);

		Vector wl;

	    boolean ok = (Argv.length > 0)
		  ? ht.setupWords(Argv[0], false, true)
		  : ht.setupWords("Delphi AND (database OR delphi)", true, true);
		if ( !ok )
		{
			System.out.println(ht.getErrorMessage());
		}

		if ( ht.search(SortByScore) < 0 )
		{
			System.out.println(ht.getErrorMessage());
		}
		else
		{
			for ( int i = 0; i < ht.getSize(); i++ )
			{
				DocMatch m = ht.get(i, true, true);
				System.out.println(m);
				System.out.println("Titel: " + m.getRef().docTitle);
				m.getTime();
				System.out.println("Size: " + m.getRef().docSize + " / " + m.getRef().docImageSize);
				//System.out.println("\nHead: " + m.getRef().docHead);
				System.out.println("\nExcerpt: " + m.getExcerpt(false));
			}
		}
	}
	catch (Exception ex)
	{
		ex.printStackTrace();
	}
  }

} // HtSearch
