/*
   Search.java -- connect the htsearch engine to webmacro framework

   $Id: Search.java,v 1.1 1999/05/24 19:41:57 eg Exp $

   Copyright (c) 1995-1999 The ht://Dig Group
   Copyright (C) 1999, Edwin Glaser

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

package htsearch;

import java.util.Hashtable;
import java.util.Vector;
import org.webmacro.util.*;
import org.webmacro.servlet.*;
import org.webmacro.engine.*;
import org.webmacro.broker.*;

public class Search extends WMServlet
{
  static Log log = new Log("search", "Search engine");
  final static private boolean debug = true;

  static Hashtable defaults = new Hashtable();
  static Hashtable sortwords = new Hashtable();
  static
  {
	defaults.put("matchesperpage", "10");
	defaults.put("page", "1");
	defaults.put("words", "");
	defaults.put("format", "long");
	defaults.put("method", "and");
	defaults.put("sort", "score");

	sortwords.put("score", new Integer(HtSearch.SortByScore));
	sortwords.put("time", new Integer(HtSearch.SortByTime));
	sortwords.put("title", new Integer(HtSearch.SortByTitle));
	sortwords.put("revscore", new Integer(HtSearch.SortByScoreRev));
	sortwords.put("revtime", new Integer(HtSearch.SortByTimeRev ));
	sortwords.put("revtitle", new Integer(HtSearch.SortByTitleRev));
  }

  String getInput(String key, WebContext context)
  {
	String res = context.getForm(key);
	if ( res == null ) res = (String) defaults.get(key);
	return res;
  }

  int getInt(String key, WebContext context)
  {
	try
	{
		return Integer.parseInt(getInput(key, context));
	}
	catch ( Exception ex )
	{
		log.warning(ex.toString());
	}
	return 0;
  }

  static Hashtable pool = new Hashtable();

  public synchronized Template handle(WebContext context) 
	throws HandlerException
  {
	// ------------ template
	String templateName = getInput("format", context);
	Template view;
	try 
	{
		view = getTemplate("htsearch/" + templateName + ".wm");
	}
	catch (ResourceUnavailableException e)
	{
		throw new HandlerException("Search loaded and executed OK, but was unable to load the template"
								   + templateName + ". \n"
								   + "Here is the actual exception that was raised:\n" + e);
	}

	// ------------ get the database connection
	String databaseName;
	try
	{
		databaseName = (String) view.getParam("database");
	}
	catch ( Exception e )
	{
		throw new HandlerException("Search was unable to get the database name. \n"
								   + "Here is the actual exception that was raised:\n" + e);
	}

	if ( databaseName == null )
	{
		throw new HandlerException("Search was unable to get the database name.");
	}

	HtSearch ht;
	try
	{
		if ( (ht = (HtSearch) pool.get(databaseName)) == null )
		{
			ht = new HtSearch (databaseName);
			pool.put(databaseName, ht);
		}
	}
	catch ( Exception e )
	{
		throw new HandlerException("Search was unable to open the database. \n"
								   + "Here is the actual exception that was raised:\n" + e);
	}

	// ------------ perform search
	String words = getInput("words", context);
	String method = getInput("method", context);
	String sort = getInput("sort", context);

	int sortby = ((Integer)sortwords.get(sort)).intValue();
 
	boolean matchBoolean = method.equals("boolean");
	boolean matchAnd = method.equals("and");

	if ( words != null && words.length() > 0 )
	{
		try
		{
			boolean ok = ht.setupWords(words, matchBoolean, matchAnd);
			if ( !ok )
			{
				throw new Exception(ht.getErrorMessage()); // FIXME errortemplate
			}
			if ( ht.search(sortby) < 0 )
			{
				throw new Exception(ht.getErrorMessage());
			}
		}
		catch ( Exception e )
		{
			throw new HandlerException("Search was unable to query the database. \n"
									   + "Here is the actual exception that was raised:\n" + e);
		}
	}

	// ------------ prepare page variables
	context.put("HTML", Html.instance());

    if (debug) log.debug("format: " + templateName);
	context.put("format", templateName);

	if (debug) log.debug("words: " + words);
	context.put("words", words);

	if (debug) log.debug("method: " + method);
	context.put("method", method);

	if (debug) log.debug("sort: " + sort);
	context.put("sort", sort);

	// ------------ number of pages
	int matchesperpage = getInt("matchesperpage", context);
	if (debug) log.debug("matchesperpage: " + matchesperpage);
	context.put("matchesperpage", getInput("matchesperpage", context));

	int page = getInt("page", context);
	if ( page < 1 ) page = 1;
	if (debug) log.debug("page: " + page);
	context.put("page", getInput("page", context));

	int pages = (ht.getSize() + matchesperpage - 1) / matchesperpage;
	String pagenumbers[] = new String[pages];
	for ( int j = 0; j < pages; j++ )
	{
		pagenumbers[j] = String.valueOf(j + 1);
	}
	context.put("pagenumbers", pagenumbers);

	int prevpage = page - 1;
	if (debug) log.debug("prevpage: " + prevpage);
	context.put("prevpage", String.valueOf(prevpage));

	int nextpage = page + 1;
	if ( nextpage > pages ) nextpage = 0;
	if (debug) log.debug("nextpage: " + nextpage);
	context.put("nextpage", String.valueOf(nextpage));

	// ------------ number of matches
	if (debug) log.debug("matches: " + ht.getSize());
	context.put("matches", String.valueOf(ht.getSize()));

	int start = (page - 1) * matchesperpage;
	if (debug) log.debug("firstdisplayed: " + (start + 1));
	context.put("firstdisplayed", String.valueOf(start + 1));

	int end = Math.min(start + matchesperpage, ht.getSize());
	if (debug) log.debug("lastdisplayed: " + end);
	context.put("lastdisplayed", String.valueOf(end));

	// ------------ the documents
	int count = end - start;
	if ( count < 0 ) count = 0;
	DocMatch[] matches = new DocMatch[count];
	for ( int i = 0; i < count; i++ )
	{
		matches[i] = ht.get(start + i, true, true);
	}
	context.put("documents", matches);

	return view;

  }


}

